function [ obj, cpu_time, gap, node ] = reformulationWithEquality( Q, u, xi, R, alpha, beta, K, T, n, seta )
% reformulationWithEquality solves a slightly modified version of Zheng
% Xiaojin's reformulation, with the cardinality inequality changed to
% an equality.


% Initiate the Cplex model

reform = Cplex();
reform.Param.simplex.limits.iterations.Cur = 1000000000;
reform.Param.timelimit.Cur = 2 * 3600;              % Set time limit
reform.Param.mip.strategy.miqcpstrat.Cur = 1;   % Use QCP relaxation at each node
reform.Param.emphasis.memory.Cur = 0;           % Reduce use of memory;
reform.Param.mip.strategy.file.Cur = 0;         % Node storage file switch;
reform.Param.output.clonelog.Cur = 0;           % Turn off clone log
reform.Param.threads.Cur = 1;                  % Disable parallel mode

reform.Model.sense='minimize';



% Fill in the data

dim = n + 4 * T;    % decision variable (x,y,phi,z), x: 1:n, y:n+1:n+T; phi:n+T+1:n+2T;
                    % z:n+2T+1:n+3T, w: n+3T+1:n+4T
                    
cstar = zeros( dim, 1 );
cstar( n + 1 : n + T, 1 ) = -R^2 * seta; % y
cstar( n + T + 1 : n + 2 * T, 1 ) = seta; % phi

lb = -inf * ones( dim, 1 ); 
lb( 1 : n + 2 * T, 1 ) = 0; % x,y,phi
ub = inf * ones( dim, 1 );
ub( 1 : n, 1 ) = u; % x
ub( n + 1 : n + T, 1 ) = 1; % y
lb( n + 3 * T + 1 : n + 4 * T, 1 ) = R;
ub( n + 3 * T + 1 : n + 4 * T, 1 ) = beta;

mytype( 1 : dim ) = 'C'; 
mytype( n + 1 : n + T ) = 'B';

% Add the linear term of the objective, lower and upper bounds of the variables
% and the types of the variables

reform.addCols( cstar, [], lb, ub, mytype ); 



% Add the quadratic term of the objective

Qstar = zeros( dim, dim );
Qstar( 1 : n, 1 : n ) = Q - xi * diag( seta ) * xi'; % x
Qstar( n + 3 * T + 1 : n + 4 * T, n + 3 * T + 1 : n + 4 * T ) = diag( seta ); % w
Qstar = Qstar + 1.e-4 * eye( dim );
Qstar = 2 * Qstar;

for i = 1 : dim
    for j = 1 : dim
        Qstar( i, j ) = Qstar( j, i );
    end
end

reform.Model.Q = Qstar;



% Add linear constraints

Ain = zeros( 2 * T, dim );
Ain( 1 : 2 * T, n + 1 : n + T ) = [ -R * eye( T ); diag( alpha ) ];
Ain( 1 : 2 * T, n + 2 * T + 1 : n + 3 * T ) = [ eye( T ); -eye( T ) ]; % -Ry_i+z_i<=0; alpha_iy_i-z_i<=0

% This is Zheng Xiaojin's original inequality
% Ain(2*T+1,n+1:n+T)=1; % e^T y <= K 

bin = zeros( 2 * T, 1 );

reform.addRows( -inf * ones( 2 * T, 1 ), Ain, bin ); % Linear inequality constraint

reform.addRows( 1, [ ones( 1, n ), zeros( 1, 4 * T ) ], 1 ); % sum(x)=1
reform.addRows( K, [ zeros( 1, n ), ones( 1, T ), zeros( 1, 3 * T ) ], K ); % e^T y == K

Aeq = zeros( T, dim ); % xi' x+R*y-z-w=0
Aeq( :, 1 : n ) = xi';
Aeq( :, n + 1 : n + T ) = R * eye( T );
Aeq( :, n + 2 * T + 1 : n + 4 * T ) = [ -eye( T ), -eye( T ) ];

% This is the equality we add
reform.addRows( zeros( T, 1 ), Aeq, zeros( T, 1 ) ); % Linear equality constraint



% Add SOCP constraints

for i = 1 : T
    aa = zeros( dim, 1 );
    Qi = zeros( dim, dim );
    Qi( 2 * T + n + i, 2 * T + n + i ) = 1;
    Qi( n + i, n + T + i ) = -0.5;
    Qi( n + T + i, n + i ) = -0.5;
    reform.addQCs( aa, Qi, 'L', 0.0 ); % Quadratic constraint: z_i^2-phi_i y_i <= 0
end % End for



% Solve the Cplex model

reform.solve();



% Ouput the results

if  isfield( reform.Solution, 'objval' )
    obj = reform.Solution.objval;
    cpu_time = reform.Solution.time;
    gap = reform.Solution.miprelgap ;
    if  isfield( reform.Solution, 'nodecnt' )
        node = reform.Solution.nodecnt;
    else
        node = 1;
    end % End if
else % "else" means the problem is infeasible
    obj = NaN;
    cpu_time = NaN;
    gap = NaN;
    node = NaN;
end

end % End reformulationWithEquality

